import debug from 'debug';
import { loggerTrace } from '../utils';
const logger = debug('teremock:driver:puppeteer:request');
let requestsCounter = 0;
export async function extractPuppeteerRequest(puppeteerRequest, options) {
    const request = {
        url: puppeteerRequest.url(),
        method: puppeteerRequest.method(),
        headers: puppeteerRequest.headers(),
        body: puppeteerRequest.postData(),
        resourceType: puppeteerRequest.resourceType(),
        id: requestsCounter++,
        timestamp: options.timestamp,
        order: options.order,
    };
    // @ts-expect-error
    puppeteerRequest.timestamp = options.timestamp;
    // @ts-expect-error
    puppeteerRequest.teremockRequest = request;
    // @ts-expect-error
    puppeteerRequest.__meta = { request };
    logger(`got the request ${request.url}, sending it to teremock core`);
    return {
        request,
        abort: async (errorCode) => await puppeteerRequest.abort(errorCode),
        next: async (interceptor) => {
            logger(`continue() call`);
            // @ts-expect-error
            puppeteerRequest.__meta.interceptor = interceptor;
            loggerTrace(`${request.url} ← request.continue()`);
            await puppeteerRequest.continue();
        },
        respond: async (response, interceptor) => {
            logger(`respond() call`);
            // @ts-expect-error
            puppeteerRequest.__meta.interceptor = interceptor;
            loggerTrace(`${request.url} ← request.respond(...)`);
            await puppeteerRequest.respond(response);
        },
    };
}
